@@ -146,6 +146,12 @@ meta_window_ensure_frame (MetaWindow *window)
   /* FIXME handle this error */
   meta_error_trap_pop (window->display);
 
+  /* Ensure focus is restored after the unmap/map events triggered
+   * by XReparentWindow().
+   */
+  if (meta_window_has_focus (window))
+    window->restore_focus_on_map = TRUE;
+
   /* stick frame to the window */
   window->frame = frame;
 
@@ -216,6 +222,12 @@ meta_window_destroy_frame (MetaWindow *window)
 
   meta_ui_destroy_frame_window (window->screen->ui, frame->xwindow);
 
+  /* Ensure focus is restored after the unmap/map events triggered
+   * by XReparentWindow().
+   */
+  if (meta_window_has_focus (window))
+    window->restore_focus_on_map = TRUE;
+
   meta_display_unregister_x_window (window->display,
                                     frame->xwindow);
 
