@@ -39,6 +39,7 @@
 #include "stack.h"
 #include "iconcache.h"
 #include <X11/Xutil.h>
+#include <X11/extensions/Xfixes.h>
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -341,14 +342,17 @@ struct _MetaWindow
   /* if TRUE, we are freezing updates during a resize */
   guint updates_frozen_for_resize : 1;
 
+  /* whether focus should be restored on map */
+  guint restore_focus_on_map : 1;
+
   /* if non-NULL, the bounds of the window frame */
   cairo_region_t *frame_bounds;
 
-  /* if non-NULL, the bounding shape region of the window */
-  cairo_region_t *shape_region;
+  /* if non-None, the bounding shape region of the window */
+  XserverRegion shape_region;
 
-  /* if non-NULL, the opaque region _NET_WM_OPAQUE_REGION */
-  cairo_region_t *opaque_region;
+  /* if non-None, the opaque region _NET_WM_OPAQUE_REGION */
+  XserverRegion opaque_region;
 
   /* _NET_WM_WINDOW_OPACITY */
   guint opacity;
